// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

//     Fichier contenant les routines de gestion frequence
//
//=======================================
// History:
// 28/10/04    V1.0
//       Version originale
//=======================================


#include "TIPLL_FreqMgt.h"
#include "TIPLL_Eeprom.h"

/* include system */
#include <avr/io.h>

/* declaration des variables static */
static  unsigned int   FREQ_CurrentFreq;  /* frequence courante */
static  unsigned char  FREQ_quickIndex;   /* index de frequence rapide */





/* fonctions applicatives de gestion EEPROM */

/*init frequence courante et index quick freq */
void FRQ_init(void)
{
    /*init frequence courante depuis l'EEPROM */
	/* ATTENTION WARNING l'EEPROM doit etre initialise AVANT l'appel de cette fonction */
	FREQ_CurrentFreq= (unsigned int)EEP_get_FreqDefault();
	FREQ_quickIndex=0;
}

char FRQ_set_currentFreq(unsigned int Frequence)   /* 0 si OK ; -1 =ereur */
{
     char Result;
	 
	 Result=0;
	 /* test de validite de la plage de frequence */
	 if ( (Frequence >= FREQ_FREQ_MIN) && (Frequence <= FREQ_FREQ_MAX))
	 {
	    FREQ_CurrentFreq=  Frequence;
	 }
	 else 
	 {
	    Result=-1;
	 }
	 
	 return (Result);
}

unsigned int  FRQ_inc_currentFreq(void)
{
    if (FREQ_CurrentFreq <= (FREQ_FREQ_MAX -FREQ_PAS_INC))
	{
	    FREQ_CurrentFreq += FREQ_PAS_INC;
	}

    return (FREQ_CurrentFreq);
}

unsigned int  FRQ_dec_currentFreq(void)
{
    if (FREQ_CurrentFreq >= (FREQ_FREQ_MIN +FREQ_PAS_INC))
	{
	    FREQ_CurrentFreq -= FREQ_PAS_INC;
	}

    return (FREQ_CurrentFreq);  
 
}

unsigned int  FRQ_inc_quickFreq(void)
{
    if (FREQ_quickIndex < (FREQ_NB_QUICK_FREQ-1))
	{
	    FREQ_quickIndex++;
	}
	else
	{
		    FREQ_quickIndex=0;
	}
	/* initialise la frequence courante avec la nouvelle valeur */
    FREQ_CurrentFreq =EEP_get_FreqQuick(FREQ_quickIndex);  
    /* retourne la valeur quick freq */
    return (FREQ_CurrentFreq);
}


unsigned int  FRQ_get_currentFreq(void)
{
    return (FREQ_CurrentFreq);  
}




