// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Fichier contenant les routines de gestion Entrees/sorties logiques
//
//=======================================
// History:
// 29/10/04    V1.0
//       Version originale
// 12/04/06    V1.1
//       Adaptation pour version ti_pfm depuis la version carte proto ti_pll
//=======================================


#include "TIPLL_IOMgt.h"
#include "TIPLL_USART2.h"   /* pour trace debug */

/* include system */
#include <avr/io.h>
#include <avr/delay.h>   /* debug temporaire */
#include <stdlib.h>

/* variables static */
static unsigned char IO_CurrentKeyb; /* etat courant du clavier */


/* fonctions applicatives de gestion IO */

/* init port E/S et valeur initiale */
void IO_init(void)
{
    /* init clavier sur portD , lock detect et LED sur PORTB */
	
	DDRD &= ~((1<<IO_BP_UP) | (1<<IO_BP_DOWN) | (1<<IO_BP_QUICK)| (1<<IO_BP_STORE));
	/* DDRD |= (1<<IO_LED_BL_LOCK); */
	DDRB |= (1<<IO_LED_LOCK);
	
	
	
	/* configuiration PORTB.0 en entree */
	DDRB &= ~(1<<IO_LOCK_DETECT);
	
	/* Config etat initial des sorties */
	PORTB &= ~(1<<IO_LED_LOCK);
	/* modif PFM PORTD |= (1<<IO_LED_BL_LOCK); */
	
    

}

/* lecture du clavier , si une touche est activee attente tempo anti rebond */
T_E_IO_CLAVIER  IO_read_keyboard(void)
{
	unsigned char KeybMask;  /* masque pour tester l'etat du clavier */
    unsigned char PreviousKeyb;            /* Etat precedent du clavier */
	
	
	T_E_IO_CLAVIER Return_Keyb=IO_NO_BP_USED;   /* par defaut aucune touche active */

	/* modif PFM ajout STORE */
	KeybMask = (1<<IO_BP_UP) | (1<<IO_BP_DOWN) | (1<<IO_BP_QUICK)| (1<<IO_BP_STORE);

	IO_CurrentKeyb= (PIND & KeybMask);


	if (IO_CurrentKeyb != KeybMask)
	{
		/* une touche est active */
		/* memorisation etat courant */
        PreviousKeyb = IO_CurrentKeyb;
		
		/* attente tempo */
		IO_wait_ms(IO_FILTRE_MS);
		/* lecture nouvel etat */
	    IO_CurrentKeyb= (PIND & KeybMask);
	
		/* SI nouvel etat identique */
		/* ALORS */
	    if (IO_CurrentKeyb == PreviousKeyb)
	    {
            /* on determine la touche active */
			/* les touches sont actives a l'etat bas donc inversion */
			/* utilisation de PreviousKeyb pour conserver l'etat courant dans IO_CurrentKeyb (autorepeat) */
			PreviousKeyb = ~PreviousKeyb ;
			/*selection des bits utilises */
			PreviousKeyb = PreviousKeyb & KeybMask;
			switch (PreviousKeyb)
			{
			    case (1<<IO_BP_UP) :
				      Return_Keyb = IO_UP;
					  break;
			     case (1<<IO_BP_DOWN) :
				      Return_Keyb = IO_DOWN;
					  break;
			    case (1<<IO_BP_QUICK) : 
				      Return_Keyb = IO_QUICK;
					  break;
				/* modif PFM ajout STORE */
				case (1<<IO_BP_STORE) : 
				      Return_Keyb = IO_STORE;
					  break;
				/* default: */
				/* si plusieurs touches actives la commande est ignoree */
				/* pas de traitement particulier */
			}
		}
        /* FINSI */
    }

	return(Return_Keyb);
}

/* retourne 0 si l'autorepeat n'est pas actif */
unsigned char IO_Keyboard_AutoRepeat(void)
{
    unsigned char KeybMask;  /* masque pour tester l'etat du clavier */
    unsigned char ReturnValue=0;
   
  	/* modif PFM ajout STORE */
	KeybMask = (1<<IO_BP_UP) | (1<<IO_BP_DOWN) | (1<<IO_BP_QUICK)| (1<<IO_BP_STORE);
	
    /* SI etat clavier identique */
    /* ALORS */
	if (IO_CurrentKeyb == (PIND & KeybMask) )
	{
        /* attente tempo autorepeat */
		IO_wait_ms(IO_FILTRE_REPEAT);
		/* SI etat clavier inchange */
		/* ALORS */
		if (IO_CurrentKeyb == (PIND & KeybMask) )
	    {
		    /* ReturnValue =1 */
			ReturnValue=1;
		/* FINSI */
		}
	/* FINSI */
	}
	
	return (ReturnValue);

}


/* lecture lock detect PLL */
T_E_IO_PLL  IO_read_LockDetect(void)
{

	T_E_IO_PLL TmpLock;
	TmpLock=IO_PLL_UNLOCK;

	/* lecture lockDetect depuis le port B */
	if (PINB && (1<<IO_LED_LOCK) == 0)
	{
        TmpLock =   IO_PLL_LOCK;    /* lock detect actif a l'etat bas */
	}
	
	return (TmpLock);
}

/* LED allumee si IO_PLL_LOCK */
void IO_set_LEDLock(T_E_IO_PLL LEDStatus)
{

	if (LEDStatus == IO_PLL_LOCK)
	{
		/* LED lock allumee (sortie active etat haut )*/
 		PORTB |= (1<<IO_LED_LOCK);
 	}
	else
	{
        /* LED lock eteinte */
  	    PORTB &= ~(1<<IO_LED_LOCK);
	}
}

/* lit l'etat du lock detect , positionne la LED en consequence et retourne cette valeur */
T_E_IO_PLL IO_refresh_LEDLock(void)
{
	T_E_IO_PLL TmpLock;
	TmpLock=IO_PLL_UNLOCK;

	/* lecture lockDetect depuis le port B */
	if ((PORTB & (1<<IO_LED_LOCK)) == 0)
	{
        TmpLock =   IO_PLL_LOCK;    /* lock detect actif a l'etat bas */
		/* LED lock allumee etat haut */
		PORTB |= (1<<IO_LED_LOCK);
	}
	else
	{
        /* LED lock eteinte */
    	PORTB &= ~(1<<IO_LED_LOCK);
	}
	
	return (TmpLock);
	
}


/* attente tempo en msec en utilisant le timer 0 */
void IO_wait_ms(unsigned int WaitTime)
{
    /* armement timer */
	/* reset du timer */

	TCNT0=1;
	/* reset flag TOV0 */
	TIFR |= (1<<TOV0);
    
	/* start timer 0 avec horloge system/1024 soit un pas de timer de 64uS pour un quartz de 16MHz */
	TCCR0 = (1<<CS02) | (1<<CS00); 
	while(WaitTime !=0)
	{
	
	    /* boucle d'attente debordement du timer correspondant  a 16.384 mS  */
		while((TIFR & (1<<TOV0)) ==0)
		{
		};
	    /* reset flag TOV0 */
        TIFR |= (1<<TOV0);
		WaitTime--;
		
	}
}





