// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.


// derived from Application note AVR306
// adaptation and modification from GL 
//=======================================
// History:
// 20/10/04    V1.0
//     first version from original application note
//=======================================

#ifndef TIPLL_USART2_H
#define TIPLL_USART2_H

/* Includes */
#include <avr/io.h>  /* definition pour ATMega8 */
#include <avr/signal.h>  /* definition pour utilisation des interruptions */
#include <avr/pgmspace.h>


/* UART Buffer Defines */
#define USART_RX_BUFFER_SIZE 128     /* 2,4,8,16,32,64,128 or 256 bytes */
#define USART_TX_BUFFER_SIZE 128     /* 2,4,8,16,32,64,128 or 256 bytes */
#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )
#define USART_TX_BUFFER_MASK ( USART_TX_BUFFER_SIZE - 1 )
#if ( USART_RX_BUFFER_SIZE & USART_RX_BUFFER_MASK )
	#error RX buffer size is not a power of 2
#endif
#if ( USART_TX_BUFFER_SIZE & USART_TX_BUFFER_MASK )
	#error TX buffer size is not a power of 2
#endif


/* Prototypes */
void USART0_Init( unsigned int baudrate );     // initialisation USART0
unsigned char USART0_Receive( void );          // Reception USART0
void USART0_Transmit( unsigned char data );    // Emission USART0
unsigned char DataInReceiveBuffer( void );     // retourne 0 si pas de donnee dans le buffer de reception

unsigned char USART_ReadCmdChar(void);         // retourne le prochain caractere avec gestion du nbr de commandes
unsigned int  USART_ReadCmdParamUi(void);      // lit un parametre Unsigned Int et le retourne apres conversion
void     USART_ReadCmdParamStr(char * ParamReadOut);     // lit une chaine delimitee par ' ' ou '\r' de car et la retourne 



void USART_PrintfStr( PGM_P data ); // Emission d'une chaine de caracteres depuis la flash
void USART_PrintfStrRAM( unsigned char  * data ); // Emission d'une chaine de caracteres
void USART_Printfc( unsigned char  Caractere );// Emission d'un caractere (surcharge applicative de USART0_Transmit)
void USART_PrintfStr_IT( unsigned char  * data ); // Emission d'une chaine de caracteres differe avec IT
void USART_Printfc_IT( unsigned char  Caractere );// Emission d'un caractere differe avec les routines d'IT
void USART0_Transmit_Poll( unsigned char data );// Emission en Polling (pour debug et mise au point)
void USART_PrintfUi_IT( unsigned int  Data, unsigned char Format ); // envoi d'un unsigned int en hexa si Format !=0


void USART_decNbCmd(void);                      // decrementation Nombre de commande
unsigned char USART_getNbCmd(void);             // retourne le nombre de commande dans le buffer


#endif
