// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

//     Fichier contenant le header des fonctions de gestion SPI
//
//=======================================
// History:
// 24/10/04    V1.0
//       Version originale
// 03/11/06    V1.1
//       ajout fonction SPI_LCD_Init
//       Correction constante d'attente LCD
// 22/11/06    V1.2
//       optimisation timing SPI
//=======================================

#ifndef TIPLL_SPI_H
#define TIPLL_SPI_H

/* declaration des fonctions master */ 
void SPI_MasterInit(void);
void SPI_MasterTransmitData(char cData);

/* declaration des fonctions slave */ 


/* definition des constantes pour le circuit PLL */

#define PLL_FREQ_REF 12000  /* en KHz --> correspond a la frequence de l'oscillateur PLL : 10MHZ*/

#define PLL_PAS_KHZ 10     /* pas de la PLL en KHz*/  
#define PLL_REG_C 0x23    /* valeur par defaut du registre C :*/
                          /* C7 POL=0 */
						  /* C6 PDA/B =0 */
						  /* C5 LDE=1 */
						  /* C4,C3,C2 par defaut OSCin/8 = 100*/
						  /* C1 fVE = 1 */
						  /* C0 fRE = 1 */
#define PLL_REG_R (PLL_FREQ_REF/PLL_PAS_KHZ)  /* valeur par defaut du registre R   */ 
#define PLL_REG_N_DEFAULT (FREQ_FREQ_MIN *((PLL_REG_R*PLL_PAS_KHZ) / PLL_FREQ_REF)) /* valeur par defaut de N */
                          /* attention l'ordre de pr?dence des operateurs * et / est important pour */
						  /* ne d?sser MAXINT et ne pas perdre la precision */

/* Remarque : La constante PLL_WAIT_ENB peut encore etre optimisee, mais l'acces a la PLL etant ponctuel*/
/* le gain serait tres faible, il est donc preferable de conserver une bonne marge, contrairement */
/* a l'afficheur LCD qui est beaucoup plus souvent utilise */
#define PLL_WAIT_ENB  20        /* temps attente PLL deselection */
#define PLL_WAIT_ENB_SELECT  8  /* temps attente PLL apres selection, avant envoi des donnees */
/* 03/11/06 constante d'attente prise en compte des commandes LCD >=250ns */
/* donc >=5 avec un Quartz 16MHz et afficheur utilise pour le debug, sinon probleme de fonctionnement avec le LCD !!!!!!!!! */
/* cette constante peut potentiellement varier selon les afficheurs utilises */
#define PLL_WAIT_LCD  8




/* Fonctions specifiques pour le circuit MC145170 */
void SPI_PLL_select(void);
void SPI_PLL_unselect(void);
void SPI_PLL_delay(unsigned int Tempo);

char SPI_setFreq(int Freq10KHZ);
void SPI_PLL_reset(void);
void SPI_PLL_init(int RegistreN);
void SPI_PLL_C(char RegistreC);
void SPI_PLL_R(int RegistreR);
void SPI_PLL_R_debug_24b(int RegistreR);
void SPI_PLL_R_debug(int RegistreR);
void SPI_PLL_N(int RegistreN);

void SPI_LCD_Init(void);
void SPI_LCD_SendData(char data);
void SPI_LCD_SendCtrl(char data);
char SPI_LCD_Read(void);
void SPI_LCD_SendLine(char * PtChar);
void SPI_LCD_Home(void);
void SPI_LCD_Clear(void);
void SPI_LCD_ClearLine1(void);
void SPI_LCD_PrintfUi(unsigned int  Data, unsigned char Format );
void SPI_LCD_DisplayCurFreq(int Freq10KHZ);
#endif

