// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// fichier principal programme de test et integration (PLL) Pilote FM
// =====================================================
// historique
//  12/04/06    V1.1
//     Version PFM derive de la version TIPLL 
//  16/10/06    V1.2
//     Correction init Baudrate pour Quartz 16MHz
//  18/10/06    V1.3
//     Correction fin du commentaire au debut de la fonction main_loop()
//  03/11/06    V1.4
//     mise en commentaires des traces de debug 
//     suppression des commandes RS232 specifiques au debug (S,L,T,I,O)
//  25/11/06    V1.5
//     ajout handler timer1 compare A pour refresh affichage init
// =====================================================



/* include system */
#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
/* pgmspace.h necessaire pour utiliser des chaines de car en Flash */
/* PSTR("string")  pour utilisation directe en flash sinon recopie en RAM !!!! */
#include <avr/pgmspace.h>   


/* include applicatif */
#include "TIPLL_USART2.h"
#include "TIPLL_spi.h"
#include "TIPLL_Eeprom.h"
#include "TIPLL_IOMgt.h"


/* define local */
#define PFM_DELAY_INIT_DISPLAY 0x8000      /* 2.09 sec pour Clk : 16MHz /1024 */


/* Application version */
const char VersionProgram[]="__@#__TI_PFM_V1.0_18/10/2006__@#__";



/* handler timer 1 pour affichage frequence initiale */
SIGNAL(SIG_OUTPUT_COMPARE1A)
{
     /*  stop timer 1*/
     TCCR1B &= ~( (1<<CS12) | (1<<CS11) | (1<<CS10)); 
     /* frequence courante display */
     SPI_LCD_DisplayCurFreq(FRQ_get_currentFreq());

      
}



void TIPLL_ReceiveMgt( void )
{
    unsigned char CurrentCar;       /* caractere courant */
    unsigned char ErreurDeSyntaxe;  /* !=0 si erreur de syntaxe */
    unsigned char IndexTmp;         /* index temporaire */
    	
    
    /* test si reception d'une commande complete */
    while ( USART_getNbCmd() !=0)
    {
	/* Process de traitement d'une commande */
	/* le format est du type : "commande parametre return" */
	/* commande est 1 car : exple : M, R, S, ...*/
	/* Parametre est un nombre ou chaine de caracteres */
	/* return est le car "\r" */
	
		ErreurDeSyntaxe=0; 
        /* SELON 1er car lu */
        CurrentCar=USART_ReadCmdChar();
        switch (CurrentCar)
        {
         	case 'M':
         	case 'm':
         	{
         		unsigned int FreqTmp;
         		/* Memorisation frequence par defaut */
         		FreqTmp=USART_ReadCmdParamUi();
         		if ((FreqTmp >= FREQ_FREQ_MIN) && (FreqTmp <=FREQ_FREQ_MAX))
         		{
         	        	USART_PrintfStr(PSTR("Memorisation Frequence:"));
         			    EEP_set_FreqDefault(FreqTmp);
						FRQ_set_currentFreq(FreqTmp); /* update frequence courante */
						SPI_setFreq(FRQ_get_currentFreq()); /* prog PLL*/
         	        	USART_PrintfUi_IT(EEP_get_FreqDefault(),0);
         	        	USART_PrintfStr(PSTR("0 KHz\r\n"));  /* affichage en KHz */
         		}
         		else
         		{
         	        	USART_PrintfStr(PSTR("\r\nERREUR frequence invalide\r\n"));
         		}
         	}
         	break;
         	

         	case 'R':
         	case 'r':
         	{
         		/* test fin de commande */
         		if(USART_ReadCmdChar() == '\r')
         		{
         		   /* Reset EEPROM */
         	           USART_PrintfStr(PSTR("Reset EEPROM"));
         	           EEP_SetDefault();
         	        }
         	        else
         	        {
         	            ErreurDeSyntaxe=1;	
         	        }
         	}
         	break;
         	
         	case 'D':
         	case 'd':
         	{
         		/* test fin de commande */
         		if(USART_ReadCmdChar() == '\r')
         		{
          			/* Display contenu EEPROM */
         	        	USART_PrintfStr(PSTR("Frequence par defaut: "));
         	        	USART_PrintfUi_IT(EEP_get_FreqDefault(),0);
         	        	USART_PrintfStr(PSTR("0 KHz\r\n"));  /* affichage en KHz */
         	        	for (IndexTmp=0; IndexTmp !=FREQ_NB_QUICK_FREQ; IndexTmp++)
         	        	{
         	           		USART_PrintfStr(PSTR("Frequence Quick["));
          	           		USART_PrintfUi_IT((unsigned int)IndexTmp,0);
         	           		USART_PrintfStr(PSTR("]: "));
          	           		USART_PrintfUi_IT(EEP_get_FreqQuick(IndexTmp),0);
         	           		USART_PrintfStr(PSTR("0 KHz\r\n"));  /* affichage en KHz */
         	        	}
         	        	USART_PrintfStr(PSTR("Flag prog freq init: "));
         	        	USART_PrintfUi_IT(EEP_get_FlagProg(),0);
         	        	USART_PrintfStr(PSTR("\r\n"));  
                	}
                	else
                	{
	       	            	ErreurDeSyntaxe=1;	
                	}
         	}
         	break;
         	
         	case 'P':
         	case 'p':
         	{
         		unsigned int FlagTmp;
         		/* Flag de programmation de la frequence initiale */
         		FlagTmp=USART_ReadCmdParamUi();
         		if (FlagTmp != 0)
         			FlagTmp=1;  /* ecretage car valeur booleen */
         			
         	        USART_PrintfStr(PSTR("Flag Programmation Freq init: "));
         		EEP_set_FlagProg(FlagTmp);
         		EEP_update_valid();
         	        USART_PrintfUi_IT(EEP_get_FlagProg(),0);
         	        USART_PrintfStr(PSTR("\r\n"));
         	}
         	break;
         	

         	case 'H':
         	case 'h':
         	case '?':
         	{
         		/* test fin de commande */
         		if(USART_ReadCmdChar() == '\r')
         		{
	         		/* Menu aide */
         	        	USART_PrintfStr(PSTR("Menu d'aide  TIPFM V1.0\r\n"));
         	        	USART_PrintfStr(PSTR("RS232 19200 Bds,8b data,2b stop, sans parite\r\n"));
         	        	USART_PrintfStr(PSTR("M Memorisation frequence en 10Khz (M 9800)\r\n"));
         	        	USART_PrintfStr(PSTR("R Reset EEPROM\r\n"));
         	        	USART_PrintfStr(PSTR("D Display EEPROM\r\n"));
         	        	USART_PrintfStr(PSTR("P Flag Prog Freq Initiale 1=prog (P 0)\r\n"));
         	        	USART_PrintfStr(PSTR("? menu d'aide \r\n"));
         	        	USART_PrintfStr(PSTR("Q Quick freq modification (Q 8900 10000 10600)\r\n"));
                 	}
                	else
                	{
	       	            	ErreurDeSyntaxe=1;	
                	}        		
         		
         	}
         	break;

         	case 'Q':
        	case 'q':
                {
         		unsigned int FreqTmp;
				unsigned char FreqValide;   /* 0 si frequence non valide */
				unsigned char IndiceFreq;   /* indice frequence*/
				
				FreqValide=1;
				IndiceFreq=0;
				
				
         		/* Quick change */
         	    USART_PrintfStr(PSTR("Modification freq Quick :\r\n"));
				
   	            /* lecture des 3 parametres */
 				while ((IndiceFreq != FREQ_NB_QUICK_FREQ) && (FreqValide !=0))
				{
         		    FreqTmp=USART_ReadCmdParamUi();
         		    if ((FreqTmp >= FREQ_FREQ_MIN) && (FreqTmp <=FREQ_FREQ_MAX))
         		    {
         	            /* erreur de syntaxe si Freq incorrecte OU parametre absent */
					    EEP_set_FreqQuick(IndiceFreq,FreqTmp);
        	           	USART_PrintfStr(PSTR("Frequence Quick["));
          	           	USART_PrintfUi_IT((unsigned int)IndiceFreq,0);
         	           	USART_PrintfStr(PSTR("]: "));
          	           	USART_PrintfUi_IT(EEP_get_FreqQuick(IndiceFreq),0);
         	           	USART_PrintfStr(PSTR("0 KHz\r\n"));  /* affichage en KHz */
         	        }
					else
					{
					   FreqValide=0;
         	           USART_PrintfStr(PSTR("\r\nERREUR frequence invalide :"));
					   USART_PrintfUi_IT(FreqTmp,0);
         	           USART_PrintfStr(PSTR("\r\n"));
					}
					IndiceFreq++;
                }   
			}
         	break;
         	default:
         	USART_PrintfStr(PSTR("\r\n\n\n\n\nErreur cmd: "));
         	USART_PrintfUi_IT(CurrentCar,1);   /* affichage commande en hexa pour filtrer les codes controles*/
         	USART_PrintfStr(PSTR(" inconnue"));
        }
        if (ErreurDeSyntaxe !=0)
        {
          	USART_PrintfStr(PSTR("\r\nErreur de syntaxe\r\n "));
        }
        USART_PrintfStr(PSTR("\r\n>")); /* affichage du prompt */
        
    }
}

void InitAppli(void)
{
    /* init I/O register */
    /*set PORTB IO_LED_LOCK pour led LOCK */
    DDRB |= (1<<IO_LED_LOCK); 
	/*  port D en entree non modifie */
	DDRC |= ( (1<<IO_LCD_RS) | (1<<IO_LCD_E) | (1<<IO_LCD_RW));  
	
	/* init USART */
	USART0_Init( 51 );   /* Set the baudrate to 19,200 bps using a 16 MHz crystal (U2X=0 0.2% erreur) */

    /* init SPI en master */
	SPI_MasterInit();
	
	/* init acces EEPROM  (imperativement avant l'init Frequence)*/
	EEP_Init();
	
	/* init gestion frequence */
	FRQ_init();

	/* init frequence courante  seulement si le flag de prog est TRUE */
	if (EEP_get_FlagProg() !=0)
	{	
	    /* init PLL MC145170 avec la frequence courante lue en EEPROM */
        SPI_PLL_init(FRQ_get_currentFreq());
	}
	
	/* init timer 1 CTC mode, OC1A interrupt set  */
	/* init OCR1A pour 1 a 2 sec */
	TCCR1B |= (1<<WGM12);  /* mode CTC OCR1A */
	OCR1A = PFM_DELAY_INIT_DISPLAY;     /* Clk : 16MHz /1024 -->15625 tick = 64s*/
	                    /* 32768 * 64s = 2.09 sec */
    /* enable interrupt  timer 1 Output compare A */
    TIMSK |= (1<<OCIE1A);
	
	
	/* autorisation IRQ */
	sei();            /* Enable interrupts => enable UART interrupts */

};


void main_loop(void)
{
    /* fonction correspondant au traitement de la boucle principale */
	unsigned char AutoRepeat=0;  /* flag autorepeat actif si different de 0 */
	T_E_IO_CLAVIER ToucheActive=IO_NO_BP_USED; /* touche active */
	
	/* 18/10/06 correction fin de commentaire ligne suivante */
	/* test clavier active */
	ToucheActive=IO_read_keyboard();
	/* SI touche active */
	/* ALORS */
	if (ToucheActive != IO_NO_BP_USED)
	{
		/* AUTOREPEAT active par defaut */
		AutoRepeat=1;
	    /* TANT QUE (touche active) OU (AUTOREPEAT actif) */
		while ((ToucheActive != IO_NO_BP_USED) && (AutoRepeat != 0))
        {		
	        /* SELON touche active */
			switch (ToucheActive)
			{
	        /* CAS UP */
			case IO_UP:
	            /* incrementation frequence */
		        //debug USART_PrintfStr(PSTR("IO_UP\r\n"));
				FRQ_set_currentFreq(FRQ_inc_currentFreq());
				SPI_setFreq(FRQ_get_currentFreq());
				break;
	        /* CAS DOWN */
			case IO_DOWN:
	            /* decrementation frequence */
				//debug USART_PrintfStr(PSTR("IO_DOWN\r\n"));
				FRQ_set_currentFreq(FRQ_dec_currentFreq());
				SPI_setFreq(FRQ_get_currentFreq());
				break;
	        /* CAS QUICK */
			case IO_QUICK:
		        /* changement frequence rapide */
				//debug USART_PrintfStr(PSTR("IO_QUICK\r\n"));
                FRQ_set_currentFreq(FRQ_inc_quickFreq());
				SPI_setFreq(FRQ_get_currentFreq());
				break;
			/* modif PFM ajout store */
	        /* CAS STORE */
			case IO_STORE:
		        /*  store frequence en EEPROM */
				 // debug USART_PrintfStr(PSTR("IO_STORE\r\n"));
        		 EEP_set_FreqDefault(FRQ_get_currentFreq());
         	     // debug USART_PrintfUi_IT(EEP_get_FreqDefault(),0);
         	     // debug USART_PrintfStr(PSTR("0 KHz\r\n"));  /* affichage en KHz */
				break;
			/* DEFAULT */
			default :
			    /* reset touche active */
				ToucheActive=IO_NO_BP_USED;
			/* FIN SELON */
			}
	        // debug USART_PrintfStr(PSTR("FREQ courante sur modif clavier : "));
            // debug USART_PrintfUi_IT(FRQ_get_currentFreq(),0);
	        // debug USART_PrintfStr(PSTR("0 KHz\r\n"));
		    /* test autorepeat actif */
			AutoRepeat = IO_Keyboard_AutoRepeat();
	    /* FIN TQ */
		}
	/* FINSI */
	}
};


int main (void)
{
   InitAppli();
   SPI_LCD_Init();
   
   /* init message + version */
   SPI_LCD_SendLine("TIPFM V1.0 init");   
   /* affichage message sur d'acceuil sur RS232 pour temps de lecture message init */
   USART_PrintfStr(PSTR("TIPFM V1.0 23/11/2006 \r\n"));
   USART_PrintfStr(PSTR("RS232 19200 Bds,8b data,2b stop, sans parite\r\n"));
   USART_PrintfStr(PSTR("\r\n"));
   USART_PrintfStr(PSTR("FREQ courante : "));
   USART_PrintfUi_IT( FRQ_get_currentFreq(),0);
   USART_PrintfStr(PSTR("0 KHz\r\n\r\n"));
   USART_PrintfStr(PSTR("? menu d'aide \r\n"));
   USART_PrintfStr(PSTR("\r\n\r\n>")); /* affichage du prompt attente nouvelle commande */
   
  /* start timer 1 pour affichage frequence au bout de ~2 sec (temps de lecture version a l'init ) */
  /* select clock /1024  ---> pas du timer 64s pour quartz 16MHz */
  TCCR1B |= ( (1<<CS12) | (1<<CS10)); 
  
   


   /* boucle tache de fond */
   while (1)
   {
     if ((PINB & (1<<IO_LOCK_DETECT)) !=0)
	 {
		 /* utilisation lock LED PORTD |= (1<<IO_LED_BL_LOCK); */
		 PORTB |= (1<<IO_LED_LOCK);
	 }
	 else
	 {
		 /* utilisation lock LED PORTD &= ~(1<<IO_LED_BL_LOCK); */
		 PORTB &= ~(1<<IO_LED_LOCK);
	 }  	  
	 TIPLL_ReceiveMgt(); /* appel fonction de gestion reception data  */
	 main_loop();

   }
return 1;
}
